#!/bin/bash
## BEGIN INIT INFO
# Provides: GRED
# Default-Start: 3 4 5
# Default-Stop: 0 1 2 3 4 6
# Required-Start: $network
#
## END INIT INFO

#chkconfig: 2354 21 79


if [[ $1 == "start" ]]; then
    set -x
fi

NAME=GRED
GRED_DIR=$(readlink -f "/opt/gred")
STATUS_FILE="${GRED_DIR}/status"
pid=`pidof -s gre_service`
NI_PRI=-16 #nice priority for GRED


wait_pid_timeout() {
    timeout=$1
    ((timeout*=100))
    c=0
    pid=$2
    until ! ps -p $pid > /dev/null || (( "$c" >= $timeout )); do
            ((c++))
            sleep 0.01
    done
}

create_pid_file() {
    timeout=$1
    ((timeout*=100))
    c=0
    pid=`pidof -s gre_service`
    ppid=`cat /proc/${pid}/status | grep ppid -i | cut -f 2 2>/dev/null`
    until [ ! -z "$pid" ] && [ "$ppid" -eq 1 ] || (( "$c" >= $timeout )); do
        ((c++))
        sleep 0.01
        pid=`pidof -s gre_service`
        ppid=`cat /proc/${pid}/status | grep ppid -i | cut -f 2`
    done

    if [ ! "$ppid" -eq 1 ] ; then
        pid=0
    fi
    echo $pid > /var/run/gred.pid
}

start() {
    if is_gred_alive ;
    then
        EXE_PATH=$(dirname $(readlink /proc/$pid/exe))
        if [[ $EXE_PATH ==  $GRED_DIR ]];
        then
            echo "ERROR: GRED already started" 
            exit 0
        else
            echo "upgrade process, kill old instance first"
            stop
        fi
    fi
    echo "Starting GRED"
    echo -e "root\t-\tcore\t" 2097152 > /etc/security/limits.conf
    echo 2 >/proc/sys/fs/suid_dumpable
    ulimit -c 2097152
    echo "/var/log/core.%e.%p.%h.%t" >  /proc/sys/kernel/core_pattern
    sh -c "cd ${GRED_DIR};export LD_LIBRARY_PATH=\"${GRED_DIR}/:\${LD_LIBRARY_PATH}\"; nice -n $NI_PRI ${GRED_DIR}/gre_service &>/dev/null &" &>/dev/null
    if [ $? = 0 ] ; then
        create_pid_file 10
        ps_aux_data=`ps aux | grep gre_service | grep -v grep`
        echo "                    [OK]"
    else
        echo "                    [FALSE]"
        exit 1
    fi
}

stop() {
    if ! is_gred_alive ; then echo -e "ERROR: GRED has not been started" && exit 0 ; fi
    echo -e "Stopping GRED($pid)"
    kill -SIGTERM $pid
    wait_pid_timeout 5 $pid
 
    # Use SIGNULL to validate PID
    kill -0 $pid 1>/dev/null 2>&1
    # Non-zero return code means the PID is dead (success)
    if [ $? -ne 0 ] ; then
        echo "                    [OK]"
    else
        echo "                    [FALSE]"
        exit 1
    fi
    pid=
}

hard_stop() {
    if ! is_gred_alive; then echo -e "ERROR: GRED has not been started" && exit 0 ; fi
    echo -e "Hard stopping GRED($pid)"
    kill -SIGQUIT $pid
    wait_pid_timeout 5 $pid
    # Use SIGNULL to validate PID
    kill -0 $pid 1>/dev/null 2>&1
    # Zero return code means the PID is still alive and should be killed forcefully
    if [ $? -eq 0 ] ; then
        kill -SIGKILL $pid &> /dev/null
    fi
    echo "                    [OK]"
    pid=
}


is_gred_alive() {
    if [[ $pid =~ ^[0-9\ ]+$ ]]; then
        return 0
    else
        return 1
    fi
}

restart() {
    echo "Restarting GRED"
    stop
    start
}

status() {
         if is_gred_alive ; then
             echo "GRED (pid $pid) is running...";
         else
             echo "GRED is stopped";
             return 1;
         fi
         cat $STATUS_FILE
}

reload() {
    if is_gred_alive ; then
        echo "Sending reconf signal to GRED (pid $pid)...";
        kill -SIGUSR1 $pid  1>/dev/null 2>&1;
        exit $?;
    else
        echo "GRED is not up yet, skipping reconf signal";
        exit 1;
    fi
}

case "$1" in
    restart)
        restart
        ;;

    start)
        start
        ;;

    stop)
        stop
        ;;

    status)
        status
        exit $?
        ;;
    reload)
        reload
        ;;
    hard_stop)
        hard_stop
        ;;
    *)
        echo $"Usage: $0 {start|stop|restart|status}"
        exit 3
        ;;
esac
